package im.zego.zpns_reactnative_sdk.internal;

import android.util.Log;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

public class ZPNsLogWriter {

    public static String TAG = "ZPNsLogWriter";

    public static void writeLog(String log, String module) {
        boolean isLoadedZPNs = false;
        try {
            System.loadLibrary("ZPNs");
            isLoadedZPNs = true;
        } catch (UnsatisfiedLinkError e) {
            e.printStackTrace();
        }

        if (!isLoadedZPNs) {
            Log.w(TAG, "ZPNs so is not loaded, ignore");
            return;
        }

        try {
            Class<?> clazz = Class.forName("im.zego.zpns.util.ZPNsLogUtils");
            Method method = clazz.getDeclaredMethod("writeCustomLog", String.class, String.class);
            method.setAccessible(true);

            String customLog = !module.isEmpty() ? "[" + module + "] " + log : log;
            method.invoke(null, customLog, "RN");
            method.setAccessible(false);
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
            Log.e(TAG, "NoSuchMethodException: " + e.getMessage());
        } catch (InvocationTargetException e) {
            String message = e.getTargetException().getMessage();
            Log.e(TAG, "new InvocationTargetException: " + message);
        } catch (IllegalAccessException e) {
            Log.e(TAG, "IllegalAccessException: " + e.getMessage(), null);
        } catch (Exception e) {
            Log.d(TAG, "onMethodCall: ");
        }
    }

    public static void printSystemProperty() {
        Log.d(TAG, "ZPNsSystemProperty: hw:" + getSystemProperty("ro.build.version.emui") + ", mi:"
                + getSystemProperty("ro.miui.ui.version.name") + ", oppo:"
                + getSystemProperty("ro.build.version.opporom") + ", vivo:" + getSystemProperty("ro.vivo.os.version"));
    }

    public static String getSystemProperty(String key) {
        try {
            Class<?> clz = Class.forName("android.os.SystemProperties");
            Method get = clz.getMethod("get", String.class, String.class);
            return (String) get.invoke(clz, key, "");
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }
}
